@@ -13,7 +13,7 @@ class _SOCKS:
     def __init__(self, proxy_endpoint):
         self._proxy_endpoint = proxy_endpoint
 
-    def hint_to_endpoint(self, hint, reactor):
+    def hint_to_endpoint(self, hint, reactor, update_status):
         mo = HINT_RE.search(hint)
         if not mo:
             raise InvalidHintError("unrecognized hint, wanted TYPE:HOST:PORT")
@@ -22,5 +22,8 @@ class _SOCKS:
         ep = SOCKS5ClientEndpoint(host, port, self._proxy_endpoint)
         return ep, host
 
+    def describe(self):
+        return "socks"
+
 def socks_endpoint(proxy_endpoint):
     return _SOCKS(proxy_endpoint)
