@@ -1,5 +1,5 @@
 
-import os, sys, time, pickle, weakref
+import os, sys, time, weakref
 import traceback
 import collections
 from twisted.python import log as twisted_log
@@ -7,7 +7,7 @@ from twisted.python import failure
 from foolscap import eventual
 from foolscap.logging.interfaces import IIncidentReporter
 from foolscap.logging.incident import IncidentQualifier, IncidentReporter
-from foolscap.logging import app_versions
+from foolscap.logging import app_versions, flogfile
 
 from foolscap.logging.levels import NOISY, OPERATIONAL, UNUSUAL, \
      INFREQUENT, CURIOUS, WEIRD, SCARY, BAD
@@ -169,17 +169,34 @@ class FoolscapLogger:
 
         if "num" not in kwargs:
             num = self.seqnum.next()
+            kwargs['num'] = num
         else:
             num = kwargs['num']
+
+        try:
+            self._msg(*args, **kwargs)
+        except Exception as e:
+            try:
+                errormsg = ("internal error in log._msg,"
+                            " args=%r, kwargs=%r, exception=%r"
+                            % (args, kwargs, e))
+                self._msg(errormsg, num=num, level=WEIRD,
+                          facility="foolscap/internal-error")
+            except:
+                pass # bummer
+        return num
+
+    def _msg(self, *args, **kwargs):
         facility = kwargs.get('facility')
         if "level" not in kwargs:
             kwargs['level'] = OPERATIONAL
         level = kwargs["level"]
         threshold = self.get_generation_threshold(facility)
         if level < threshold:
-            return num # not worth logging
+            return # not worth logging
 
         event = kwargs
+        # kwargs always has 'num'
 
         if "format" in event:
             pass
@@ -212,18 +229,10 @@ class FoolscapLogger:
                 f2.setCopyableState(fs.getStateToCopy(f, FakeBroker))
                 event["failure"] = f2
 
-        # verify that we can stringify the event correctly
-        try:
-            format_message(event)
-        except Exception, e:
-            print "problem in log message %s: %r %s" % (event, e, e)
-            pass
         if event.get('stacktrace', False) is True:
             event['stacktrace'] = traceback.format_stack()
         event['incarnation'] = self.incarnation
-        event['num'] = num
         self.add_event(facility, level, event)
-        return num
 
     def err(self, _stuff=None, _why=None, **kw):
         """
@@ -275,11 +284,8 @@ class FoolscapLogger:
         # eventual-send.
 
         if self.active_incident_qualifier:
-            try:
-                # this might call declare_incident
-                self.active_incident_qualifier.event(event)
-            except:
-                print failure.Failure() # for debugging
+            # this might call declare_incident
+            self.active_incident_qualifier.event(event)
 
     def declare_incident(self, triggering_event):
         self.incidents_declared += 1
@@ -343,16 +349,13 @@ class TwistedLogBridge:
         self.tubID = tubID
         self.logger = foolscap_logger
 
-        # newer versions of Twisted have a function called
-        # textFromEventDict() that we can use to format the message. Older
-        # versions (twisted-2.5.0 and earlier) have this functionality buried
-        # in the FileLogObserver where it isn't very easy to get to.
-        if hasattr(twisted_log, "textFromEventDict"):
-            self.observer = self._new_twisted_log_observer
-        else:
-            self.observer = self._old_twisted_log_observer
+    # we currently depend on Twisted >= 10.1.0, so we can use
+    # t.p.log.textFromEventDict . However we cannot add ourselves as a
+    # new-style observer (t.l.globalLogPublisher.addObserver()) because that
+    # wasn't added until 15.2.0. So even on newer Twisteds, we'll be wrapped
+    # by t.l._legacy.LegacyLogObserverWrapper
 
-    def _new_twisted_log_observer(self, d):
+    def observer(self, d):
         # Twisted will remove this for us if it fails.
         if "from-foolscap" in d:
             return
@@ -362,56 +365,42 @@ class TwistedLogBridge:
         #  log.err() adds: isError=True, failure, why
         # plus any kwargs provided to msg()/err(), like format=
 
-        # Twisted-15.2.0 introduces a new logging system, and adds several
-        # new log_* keys to the observed event dictionary, two of which
-        # (log_level and log_legacy) are non-serializable. Convert all the
-        # new keys into foolscap-style values, or delete them altogether.
+        # With Twisted-15.2.0 we are wrapped by
+        # t.l._legacy.LegacyLogObserverWrapper , so we still get those keys,
+        # but we'll also see some log_* keys that the new logging system
+        # adds. Some of the new keys are non-serializable.
 
+        # So we stringify the Twisted event right now, and produce a new
+        # event with a small set of known keys.
+
+        message = twisted_log.textFromEventDict(d)
+        kwargs = {'tubID': self.tubID, 'from-twisted': True}
+
+        # log_level was added in 15.2.0
         if "log_level" in d:
-            # d["log_level"] is a non-serializable ConstantString. Transform
-            # it into the corresponding (integer) Foolscap log level, and
-            # omit the original key.
+            # d["log_level"] might be a non-serializable ConstantString.
+            # Transform it into the corresponding (integer) Foolscap log
+            # level.
             log_level = d.pop("log_level")
             new_log_level = llmap.get(log_level, log_level)
-            if isinstance(new_log_level, (int, long)):
-                d["level"] = new_log_level # foolscap level, not twisted level
-            else:
+            if not isinstance(new_log_level, (int, long, str, unicode, bool)):
                 # it was something weird: just stringify it in-place
-                if not isinstance(new_log_level, (str, unicode, bool)):
-                    new_log_level = str(new_log_level)
-                d["log_level"] = new_log_level # twisted level
+                new_log_level = str(new_log_level)
+            kwargs["level"] = new_log_level # foolscap level, not twisted
 
-        # Normal log.msg(foo) results in d["format"]="%(log_legacy)s" and a
-        # non-serializable d["log_legacy"] (which is meant to defer
-        # stringification).
-        if d.get("format") == "%(log_legacy)s":
-            # just get rid of format and log_legacy, since they'll give the
-            # same thing as textFromEventDict()
-            d.pop("format")
-            d.pop("log_legacy", None)
-        event = d.copy()
-        event['tubID'] = self.tubID
-        message = twisted_log.textFromEventDict(d)
-        event.pop('message', None)
-        event['from-twisted'] = True
-        self.logger.msg(message, **event)
+        # d["isError"]=1 for pre-15.2.0 calls to t.p.log.err(), and is
+        # synthesized by the LegacyLogObserverWrapper for post-15.2.0 calls
+        # when the event includes a Failure or a log_level of "error" or
+        # "critical". In post-15.2.0 calls, "time" and "system" are copied
+        # from log_time and log_system, and "log_namespace" seems pretty
+        # useful.
+        for k in ["isError", "why", "time", "system", "log_namespace"]:
+            if k in d:
+                kwargs[k] = d[k]
+        # we don't copy d["failure"] or d["why"], because its text should
+        # already be copied into "message".
 
-    def _old_twisted_log_observer(self, d):
-        if "from-foolscap" in d:
-            return
-        event = d.copy()
-        if "format" in d:
-            # 'message' will be treated as a format string, with the rest of
-            # the arguments as its % dictionary.
-            pass
-        else:
-            # no string interpolation, but if there are multiple args, then
-            # join them together, to match what twisted does.
-            event['message'] = " ".join([str(m) for m in d['message']])
-        event.pop('args', None)
-        event['tubID'] = self.tubID
-        event['from-twisted'] = True
-        self.logger.msg(**event)
+        self.logger.msg(message, **kwargs)
 
 _bridges = {} # maps (twisted_logger,foolscap_logger) to TwistedLogBridge
 
@@ -476,12 +465,11 @@ class LogFileObserver:
         else:
             self._logFile = open(filename, "wb")
         self._level = level
-        header = {"header": {"type": "log-file-observer",
-                             "threshold": level,
-                             "versions": app_versions.versions,
-                             "pid": os.getpid(),
-                             }}
-        pickle.dump(header, self._logFile)
+        flogfile.serialize_header(self._logFile,
+                                  "log-file-observer",
+                                  versions=app_versions.versions,
+                                  pid=os.getpid(),
+                                  threshold=level)
 
     def stop_on_shutdown(self):
         from twisted.internet import reactor
@@ -492,11 +480,8 @@ class LogFileObserver:
         #if event.get('facility', '').startswith('foolscap'):
         #    threshold = UNUSUAL
         if event['level'] >= threshold:
-            e = {"from": "local",
-                 "rx_time": time.time(),
-                 "d": event,
-                 }
-            pickle.dump(e, self._logFile, 2)
+            flogfile.serialize_wrapper(self._logFile, event,
+                                       from_="local", rx_time=time.time())
 
     def _stop(self):
         self._logFile.close()
